/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   Data Writing Driver                                              */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT+ Ver1.00`                                                    */
/*  FileID :   23c                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

package hldc.dt10.driver;

import java.io.*;

public class DtTestPointDriver {
    
    //-------------------------------------------------------------------------
    //  DynamicTracar MaskCode
    //-------------------------------------------------------------------------
    static public final int DT_VARIABLE_BIT       =    0x02;                /* Variable MASK */
    static public final int DT_EVTTRG_BIT         =    0x08;                /* EventTrigger MASK */

    //-------------------------------------------------------------------------
    //  Please set true when you add Event Value to Test Point
    //-------------------------------------------------------------------------
    static public boolean DT_USE_EVENT_INFO       =    true;
    
    static public File file;
//    static public FileOutputStream out;
    static public BufferedOutputStream out;
    
    static public long timeCount, startTime;
    static public long thread_id = 0;
    static byte[] buf = new byte[16];

    //-------------------------------------------------------------------------
    //  TestPoint InsertCode Method
    //-------------------------------------------------------------------------
    /* Normal TestPoint Method */
    static public synchronized final void DtTestPoint(int nBaseAddress, int nStepNumber){
        TpBusOut( nBaseAddress, nStepNumber);
    }

    /* MemoryOutput TestPoint Method */
    static public synchronized final void DtTestPointValue(int nBaseAddress, int nStepNumber, byte theValue, int nLength){
        /* Call DynamicTracerDriver */
        TpBusOut( nBaseAddress | DT_VARIABLE_BIT , nStepNumber );        /* Memory Dump Start */ 

        buf[0] = 1;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = 0x00;
        buf[6] = 0x00;
        buf[7] = 0x00;
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try {
            out.write(buf);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static public synchronized final void DtTestPointValue(int nBaseAddress, int nStepNumber, char theValue, int nLength){
        /* Call DynamicTracerDriver */
        TpBusOut( nBaseAddress | DT_VARIABLE_BIT , nStepNumber );        /* Memory Dump Start */

        buf[0] = 2;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = (byte)((theValue >> 8) & 0xff);
        buf[6] = 0x00;
        buf[7] = 0x00;
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try {
            out.write(buf);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static public synchronized final void DtTestPointValue(int nBaseAddress, int nStepNumber, short theValue, int nLength){
        /* Call DynamicTracerDriver */
        TpBusOut( nBaseAddress | DT_VARIABLE_BIT , nStepNumber );        /* Memory Dump Start */

        buf[0] = 2;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = (byte)((theValue >> 8) & 0xff);
        buf[6] = 0x00;
        buf[7] = 0x00;
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try {
            out.write(buf);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static public synchronized final void DtTestPointValue(int nBaseAddress, int nStepNumber, int theValue, int nLength){
        /* Call DynamicTracerDriver */
        TpBusOut( nBaseAddress | DT_VARIABLE_BIT , nStepNumber );        /* Memory Dump Start */

        buf[0] = 4;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = (byte)((theValue >> 8) & 0xff);
        buf[6] = (byte)((theValue >> 16) & 0xff);
        buf[7] = (byte)((theValue >> 24) & 0xff);
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try {
            out.write(buf);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static public synchronized final void DtTestPointValue(int nBaseAddress, int nStepNumber, long theValue, int nLength){
        /* Call DynamicTracerDriver */
        TpBusOut( nBaseAddress | DT_VARIABLE_BIT , nStepNumber );        /* Memory Dump Start */

        buf[0] = 8;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = (byte)((theValue >> 8) & 0xff);
        buf[6] = (byte)((theValue >> 16) & 0xff);
        buf[7] = (byte)((theValue >> 24) & 0xff);
        buf[8] = (byte)((theValue >> 32) & 0xff);
        buf[9] = (byte)((theValue >> 40) & 0xff);
        buf[10] = (byte)((theValue >> 48) & 0xff);
        buf[11] = (byte)((theValue >> 56) & 0xff);
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try {
            out.write(buf);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static public synchronized final void DtTestPointValue(int nBaseAddress, int nStepNumber, boolean theValue, int nLength){
        int val = theValue?0x10:0x00;
        /* Call DynamicTracerDriver */
        TpBusOut( nBaseAddress | DT_VARIABLE_BIT , nStepNumber );        /* Memory Dump Start */

        buf[0] = 2;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)(val & 0xff);
        buf[5] = (byte)((val >> 8) & 0xff);
        buf[6] = 0x00;
        buf[7] = 0x00;
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try {
            out.write(buf);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static public synchronized final void DtTestPointValue(int nBaseAddress, int nStepNumber, byte[] theValue, int nLength){
        int packetNum;
        int value_num;
        /* Call DynamicTracerDriver */
        if( nLength > 0 ){
            TpBusOut( nBaseAddress | DT_VARIABLE_BIT , nStepNumber );        /* Memory Dump Start */

            packetNum = (nLength + 7) / 8;

            buf[0] = (byte)nLength;       /* size */
            buf[1] = (byte)packetNum;     /* packetNum */
            buf[2] = 0x03;
            buf[12] = 0x00;
            buf[13] = 0x00;
            buf[14] = 0x00;
            buf[15] = 0x00;
            for(int cnt = 0; cnt < packetNum; cnt++){
                value_num = (cnt * 8);
                buf[3] = (byte)cnt;
                for (int i = 0; i < 8; i++) {
                    buf[4+i] = (byte)((theValue[value_num++]) & 0xff);
                }
                try {
                    out.write(buf);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /* Normal TestPoint Method */
    static public final void TpBusOut(int nAddress, int nData){
        if (out == null) {
            openFile();
        }

        /* time data */
        timeCount = System.nanoTime() - startTime;
        /* thread ID */
        thread_id = Thread.currentThread().getId();

        buf[0] = (byte)((nData) & 0xff);
        buf[1] = (byte)((nData >> 8) & 0xff);
        buf[2] = (byte)((nAddress) & 0xff);
        buf[3] = (byte)((nAddress >> 8) & 0xff);
        buf[4] = (byte)((nAddress >> 16) & 0xff);
        
        if (DT_USE_EVENT_INFO) {
            buf[5] = (byte)((timeCount) & 0xff);
            buf[6] = (byte)((timeCount >> 8) & 0xff);
            buf[7] = (byte)((timeCount >> 16) & 0xff);
            buf[8] = (byte)((timeCount >> 24) & 0xff);
            buf[9] = (byte)((thread_id) & 0xff);
            buf[10] = (byte)((thread_id >> 8) & 0xff);
            buf[11] = (byte)((thread_id >> 16) & 0xff);
            buf[12] = (byte)((thread_id >> 24) & 0xff);
        } else {
            buf[5] = (byte)((timeCount) & 0xff);
            buf[6] = (byte)((timeCount >> 8) & 0xff);
            buf[7] = (byte)((timeCount >> 16) & 0xff);
            buf[8] = (byte)((timeCount >> 24) & 0xff);
            buf[9] = (byte)((timeCount >> 32) & 0xff);
            buf[10] = (byte)((timeCount >> 40) & 0xff);
            buf[11] = (byte)((timeCount >> 48) & 0xff);
            buf[12] = (byte)((timeCount >> 56) & 0xff);
        }
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        
        try {
            out.write(buf);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /* open file */
    static public void openFile() {
        file = new File("Convertdata.DAT");
        try {
            out = new BufferedOutputStream(new FileOutputStream(file));
            startTime = System.nanoTime();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /* close file */
    static public void closeFile() {
        try {
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}

